<?php
/**
 * @category    Pinpoint
 * @package     Pinpoint_RegisterOnSuccess
 * @copyright   Copyright (c) 2019 Pinpoint Designs. (http://www.pinpointdesigns.co.uk)
 * @author      Mohamed Zahran <mohamed.zahran@bytology.co.uk>
 */

namespace Pinpoint\RegisterOnSuccess\Block;

use Magento\Customer\Model\AccountManagement;
use Magento\Newsletter\Observer\PredispatchNewsletterObserver;

/**
 * Class Register
 */
class Register extends \Magento\Framework\View\Element\Template
{
    const CMS_BLOCK_ONE_TITLE = 'REGISTER ON SUCCESS CMS BLOCK ONE';
    const CMS_BLOCK_ONE_IDENTIFIER = 'register_on_success_cms_block_one';

    const CMS_BLOCK_TWO_TITLE = 'REGISTER ON SUCCESS CMS BLOCK TWO';
    const CMS_BLOCK_TWO_IDENTIFIER = 'register_on_success_cms_block_two';

    const XML_PATH_IS_MODULE_ENABLED = 'customer/register_on_success/enabled';
    const XML_PATH_SECTION_TITLE = 'customer/register_on_success/section_title';

    const XML_PATH_REDIRECT_TO_ACCOUNT_DASHBOARD = 'customer/register_on_success/redirect_to_account_dashboard';

    protected $_template = 'Pinpoint_RegisterOnSuccess::register.phtml';

    /**
     * @var \Magento\Checkout\Model\Session
     */
    protected $checkoutSession;

    /**
     * @var \Magento\Customer\Model\Session
     */
    protected $customerSession;

    /**
     * @var \Magento\Customer\Model\Registration
     */
    protected $registration;

    /**
     * @var \Magento\Customer\Api\AccountManagementInterface
     */
    protected $accountManagement;

    /**
     * @var \Magento\Sales\Api\OrderRepositoryInterface
     */
    protected $orderRepository;

    /**
     * @var \Magento\Sales\Model\Order\Address\Validator
     */
    protected $addressValidator;

    /**
     * @var \Magento\Customer\Model\Url
     */
    protected $customerUrl;

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var \Magento\Framework\Module\Manager
     */
    protected $moduleManager;

    /**
     * @param \Magento\Framework\View\Element\Template\Context $context
     * @param \Magento\Checkout\Model\Session $checkoutSession
     * @param \Magento\Customer\Model\Session $customerSession
     * @param \Magento\Customer\Model\Registration $registration
     * @param \Magento\Customer\Api\AccountManagementInterface $accountManagement
     * @param \Magento\Sales\Api\OrderRepositoryInterface $orderRepository
     * @param \Magento\Sales\Model\Order\Address\Validator $addressValidator
     * @param \Magento\Customer\Model\Url $customerUrl
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Magento\Framework\Module\Manager $moduleManager
     * @param array $data
     * @throws \Magento\Framework\Exception\LocalizedException
     * @codeCoverageIgnore
     */
    public function __construct(
        \Magento\Framework\View\Element\Template\Context $context,
        \Magento\Checkout\Model\Session $checkoutSession,
        \Magento\Customer\Model\Session $customerSession,
        \Magento\Customer\Model\Registration $registration,
        \Magento\Customer\Api\AccountManagementInterface $accountManagement,
        \Magento\Sales\Api\OrderRepositoryInterface $orderRepository,
        \Magento\Sales\Model\Order\Address\Validator $addressValidator,
        \Magento\Customer\Model\Url $customerUrl,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Framework\Module\Manager $moduleManager,
        array $data = []
    ) {
        $this->checkoutSession = $checkoutSession;
        $this->customerSession = $customerSession;
        $this->registration = $registration;
        $this->accountManagement = $accountManagement;
        $this->orderRepository = $orderRepository;
        $this->addressValidator = $addressValidator;
        $this->customerUrl = $customerUrl;
        $this->scopeConfig = $scopeConfig;
        $this->moduleManager = $moduleManager;
        parent::__construct($context, $data);
        /** Replace the checkout registration button with our module */
        if ($this->isModuleEnabled()) {
            $this->getLayout()->unsetElement('checkout.registration');
        }
    }

    /**
     * Retrieve the block title
     *
     * @return string
     */
    public function getTitle()
    {
        return $this->scopeConfig->getValue(
            self::XML_PATH_SECTION_TITLE
        );
    }

    /**
     * Check if the module is enabled
     *
     * @return bool
     */
    public function isModuleEnabled()
    {
        return (bool)$this->scopeConfig->getValue(
            self::XML_PATH_IS_MODULE_ENABLED
        );
    }

    /**
     * Check if the customer should be redirected to account page after registration.
     *
     * @return bool
     */
    public function shouldRedirectToAccountDashboard()
    {
        return (bool)$this->scopeConfig->getValue(
            self::XML_PATH_REDIRECT_TO_ACCOUNT_DASHBOARD
        );
    }

    /**
     * @return \Magento\Sales\Model\Order
     */
    public function getOrder()
    {
        return $this->checkoutSession->getLastRealOrder();
    }

    /**
     * Retrieve form posting url
     *
     * @return string
     */
    public function getPostActionUrl()
    {
        return $this->customerUrl->getRegisterPostUrl();
    }

    /**
     * Get minimum password length
     *
     * @return string
     */
    public function getMinimumPasswordLength()
    {
        return $this->scopeConfig->getValue(
            AccountManagement::XML_PATH_MINIMUM_PASSWORD_LENGTH
        );
    }

    /**
     * Get number of password required character classes
     *
     * @return string
     */
    public function getRequiredCharacterClassesNumber()
    {
        return $this->scopeConfig->getValue(
            AccountManagement::XML_PATH_REQUIRED_CHARACTER_CLASSES_NUMBER
        );
    }

    /**
     * Newsletter module availability
     *
     * @return bool
     */
    public function isNewsletterEnabled()
    {
        return $this->moduleManager->isOutputEnabled('Magento_Newsletter')
            && $this->scopeConfig->getValue(
                PredispatchNewsletterObserver::XML_PATH_NEWSLETTER_ACTIVE
            );
    }

    /**
     * Retrieve form data
     *
     * @return mixed
     */
    public function getFormData()
    {
        $data = $this->getData('form_data');
        if ($data === null) {
            $formData = $this->customerSession->getCustomerFormData(true);
            $data = new \Magento\Framework\DataObject();
            if ($formData) {
                $data->addData($formData);
                $data->setCustomerData(1);
            }
            $this->setData('form_data', $data);
        }
        return $data;
    }

    /**
     * Retrieve current email address
     *
     * @return string
     */
    public function getCustomerEmailAddress()
    {
        return $this->checkoutSession->getLastRealOrder()->getCustomerEmail();
    }

    /**
     * Retrieve customer first name
     *
     * @return string|null
     */
    public function getCustomerFirstname()
    {
        return $this->checkoutSession->getLastRealOrder()->getCustomerFirstname();
    }

    /**
     * Retrieve customer last name
     *
     * @return string|null
     */
    public function getCustomerLastname()
    {
        return $this->checkoutSession->getLastRealOrder()->getCustomerLastname();
    }

    /**
     * {@inheritdoc}
     */
    public function toHtml()
    {
        if ($this->customerSession->isLoggedIn()
            || !$this->registration->isAllowed()
            || !$this->accountManagement->isEmailAvailable($this->getCustomerEmailAddress())
            || !$this->validateAddresses()
            || !$this->isModuleEnabled()
        ) {
            return '';
        }
        return parent::toHtml();
    }

    /**
     * Validate order addresses
     *
     * @return bool
     */
    protected function validateAddresses()
    {
        $lastOrderId = $this->checkoutSession->getLastOrderId();
        $order = $this->orderRepository->get($lastOrderId);
        $addresses = $order->getAddresses();
        foreach ($addresses as $address) {
            $result = $this->addressValidator->validateForCustomer($address);
            if (is_array($result) && !empty($result)) {
                return false;
            }
        }
        return true;
    }

    /**
     * Retrieve the content of CMS Block One
     *
     * @return mixed
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function getCmsBlockOne()
    {
        return $this
            ->getLayout()
            ->createBlock(\Magento\Cms\Block\Block::class)
            ->setBlockId(self::CMS_BLOCK_ONE_IDENTIFIER)
            ->toHtml();
    }

    /**
     * Retrieve the content of CMS Block Two
     *
     * @return mixed
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function getCmsBlockTwo()
    {
        return $this
            ->getLayout()
            ->createBlock(\Magento\Cms\Block\Block::class)
            ->setBlockId(self::CMS_BLOCK_TWO_IDENTIFIER)
            ->toHtml();
    }
}
